%% extracted from: Xuchun Li, Lei Wang, Eric Sung - "A Study of AdaBoost with SVM Based Weak Learners" - IJCNN 2005

function [Accuracy Specificity Sensitivity] = ProbabilisticNN_AdaBoost_Varying_Spread ()

   load datos_vbm_procesados_para_neuralnets.mat

   C = voxels_matrix_controles;
   P = voxels_matrix_pacientes;

   k_de_kfold = 10;
   p = cvpartition(size(C,1),'kfold',k_de_kfold);
   c = cvpartition(size(C,1),'kfold',k_de_kfold);

   patient_label     = 1;
   control_label     = 2;

   spread_coeff_min  = 0.1;
   spread_coeff_step = 0.1;
   spread_coeff_top  = 1;
   spread_coeff      = spread_coeff_min;

   %weights for the classifier of each cluster, will be used for the final adaboost classifier
   num_of_weaklearners = (spread_coeff_top - spread_coeff_min)/spread_coeff_step;
   alpha = ones(1,num_of_weaklearners)/num_of_weaklearners;

   %start training adaboost cluster SVM
   for k=1:k_de_kfold;

      %compute folds
      %training set used to compute the weights for each clusterSVM (our weaklearners)
      C_training = C(training (c,k) == 1,:);
      P_training = P(training (p,k) == 1,:);

      %test set used to validate the adaboost
      C_test     = C(training (c,k) == 0,:);
      P_test     = P(training (p,k) == 0,:);

      training_patients_no = size(P_training,1);
      training_controls_no = size(C_training,1);
      training_subjects_no = training_patients_no + training_controls_no;
      sample_weight = 1/training_subjects_no;

      %weight training
      for i=1:num_of_weaklearners;
         %training
         [weight_training_data(k,i).data, weight_training_data(k,i).label] = alignAndLabelDataForNN (C_training, P_training, control_label, patient_label);

         label_ind = ind2vec(weight_training_data(k,i).label);

         %creating neural network and training with training set
         weaklearners(k,i).net = newpnn (weight_training_data(k,i).data', label_ind, spread_coeff);

         %classification test with training data
         weaklearners(k,i).training_output = vec2ind ( sim (weaklearners (k,i).net, weight_training_data(k,i).data') );

         %adaboost compute training errors
         training_errors(k,i) = sample_weight;
         for s=1:size(weight_training_data(k,i).data,1);
            if (weaklearners (k,i).training_output(s) ~= weight_training_data(k,i).label(s));
               training_errors(k,i) = training_errors(k,i) + sample_weight;
            end;
         end;

         %adaboost set classifier weights
         alpha (i) = 0.5 * log((1-training_errors(k,i)) / training_errors(k,i));

         spread_coeff = spread_coeff + spread_coeff_step;
      end;

      %classify test set
      for i=1:num_of_weaklearners;
         [test_data(k,i).data, test_data(k,i).label] = alignAndLabelDataForNN (C_test, P_test, control_label, patient_label);

         label_ind = ind2vec(test_data(k,i).label);

         boosted_results(k,i).output = vec2ind ( sim (weaklearners (k,i).net, test_data(k,i).data') );

         %write test data to text
         %path=strcat('cluster',num2str(i),'_testdata_', num2str(k),'.txt');
         %writeDataToLIBSVMFormat (path, test_data(i).data, test_data(i).label);
      end;

      %classify and evaluate adaboost
      total_tests = 0;
      TP = 0;
      TN = 0;
      FP = 0;
      FN = 0;

      for s=1:size(test_data(1).data,1);
         %organize outputs for one subject in one vector
         subj_class = 0;
         subj_outputs = zeros(1,num_of_weaklearners);
         for c=1:num_of_weaklearners;
            subj_outputs(c) = boosted_results (k,c).output(s);
         end;

         %change the label values from 0 1 or 1 2 to -1 1 in order to 
         %make the adaboost result computation easier (or possible?)
         subj_outputs  = (subj_outputs==control_label) + (subj_outputs==patient_label)*(-1);
         target_labels = (test_data(1).label==control_label) + (test_data(1).label==patient_label)*(-1);

         %adaboost classification with alpha (weaklearner weights)
         %if negative: class = -1
         %else if positive: class = 1
         %else if zero: only the first cluster SVM prediction with maximum weight will be taken into account  
         subj_class = sum(alpha.*subj_outputs);
         if (subj_class == 0);
            [max_value max_idx] = max(alpha);
            subj_class = subj_outputs (max_idx(1));
         end;

         subj_predictions(s) = sign(subj_class);

         if (target_labels(s) == -1); %positive
            if   (subj_predictions(s) == -1); TP = TP + 1;
            else                              FN = FN + 1;
            end;
         else %negative
            if   (subj_predictions(s) ==  1); TN = TN + 1;
            else                              FP = FP + 1;
            end;
         end;

         total_tests = total_tests + 1;

      end;

   %   predictions(k,:) = subj_predictions(:);

      clear subj_predictions;

   end;

   Accuracy    = (TP + TN) / total_tests;
   Specificity = TN / (TN+FP);
   Sensitivity = TP / (TP+FN);


